import { _decorator, Component, Node, Game } from "cc";
import BaseData from "./BaseData";
import global from "./global";
import GDef from "./GDef";
import { Notifications } from "../easyFramework/notifications";
import { MathRandom } from "../utils/UtilsMath";
import DataManager from "./DataManager";
const { ccclass, property } = _decorator;

const CONFIGS = {
    cacheName: "levelInfo"
}
interface cacheData {
    /**当前关卡 */
    Checkpoint: number;
}
//关卡数据管理
class levelMgr extends BaseData {//字段可以直接读,改动最好通过接口
    /**后台数据 */
    dictCache!: cacheData;
    /**游戏状态 */
    constructor() {
        super(CONFIGS.cacheName);
    }
    init() {
        let info: cacheData = this.getCache();
        if (info) {
            this.dictCache = info;
        } else {
            this.dataInit();
        }
    }

    /**数据初始化 */
    dataInit(): void {
        this.dictCache = {
            Checkpoint: 1
        };
        this.updateCache();
    }

    /**
     * 获取当前关卡
     */
    get checkPoint(): number {
        return this.dictCache.Checkpoint;
    }

    /**
     * 当前关卡+1
     */
    setCheckpoint(): void {
        this.dictCache.Checkpoint++;
        this.updateCache();
    }

    /**获取关卡数据
     * @param ID 如果传进关卡ID，则指定加载某一关
     */
    getCheckJson(ID?: number): Object {
        var num = 0;
        if (ID) {
            num = ID;
        } else {
            num = this.dictCache.Checkpoint;
            //超过关卡上限，随机一关出来
            if (num > global.levelCount) {
                num = MathRandom.getInstance().RandomNumBoth(1, global.levelCount);
            }
        }
        const lv = "level" + num;
        return DataManager.getTable(lv);
    }
}

export default levelMgr;