import { _decorator, Component, Node, Game, game } from "cc";
import BaseData from "./BaseData";
import global from "../config/global";
import GDef from "./GDef";
import { Notifications } from "../easyFramework/notifications";
import GD from "./GD";
const { ccclass, property } = _decorator;

const CONFIGS = {
    cacheName: "playerInfo"
}
interface cacheData {
    /**金币 */
    coin: number,
    /**体力 */
    physical: number,
    /**时间 */
    curTime: number,
    /**判断新旧玩家 新玩家new 老玩家old*/
    player: string,
    /**震动  1表示开启震动 2表示关闭震动*/
    shock: number,
    /**声音 1表示开启声音 2表示关闭声音*/
    voice: number,
    /**vip观看广告次数*/
    vipCount: number,
    /**开启VIP的时间 */
    vipStarTime: number,
    /**地狱模式1看视频数量  */
    specialModelCount_1: number,
    /**天选模式看视频数量 */
    specialModelCount_2: number,
    /**弹窗1为无敌状态,2为皮肤适用 */
    popupState:number
}
//玩家数据管理
class playerMgr extends BaseData {//字段可以直接读,改动最好通过接口
    /**后台数据 */
    dictCache!: cacheData;
    /**游戏状态 */
    gameState!: number
    private isCutDown = false;//是否正在倒计时
    constructor() {
        super(CONFIGS.cacheName);
    }
    init() {
        console.log("初始化玩家管理.........");
        let info: cacheData = this.getCache();
        if (info) {
            this.dictCache = info;
            const date = new Date();
            var curTime = date.getTime();
            this.afterTime(curTime);
            this.afterTimeVip(curTime);
        } else {
            this.dataInit();
        }
        this.gameState = global.GAME_STATE.GAME_WAIT//游戏变成等待
        game.on(Game.EVENT_HIDE, () => {
            this.setTime();
        });
    }

    /**数据初始化 */
    dataInit() {
        const date = new Date();
        var curTime = date.getTime();
        this.dictCache = {
            coin: 0,
            physical: global.Physical,
            curTime: curTime,
            player: "",
            shock: 1,
            voice: 1,
            vipCount: 0,
            vipStarTime: 0,
            specialModelCount_1: 0,
            specialModelCount_2: 0,
            popupState:1
        };
        this.updateCache();
    }

    /**
     * 经过多少时间(体力)
     */
    afterTime(curTime: number) {
        let count = Number(this.dictCache.physical);
        if (count >= global.Physical) return;
        var lastTime = Number(this.dictCache.curTime);
        var minute = 0;
        minute = (curTime - lastTime) / 1000 / 60 / global.physicalTime;
        const num = Math.floor(minute);
        this.dictCache.curTime = lastTime;//当前时间是上一次退出的时间，用于区分倒计时
        if (this.dictCache.curTime) {
            const time = global.physicalTime * 60 - Math.floor((curTime - this.dictCache.curTime) / 1000 % (global.physicalTime * 60));
            if (time > 0) {
                this.recoveryPhysical(time);
            }
        }
        if (num > 0) {
            this.setPhysical(num * global.physicalCount);
        }
    }

    /**经过多少时间 (vip时间)*/
    afterTimeVip(curTime: number) {
        if (!this.dictCache.vipStarTime || this.dictCache.vipStarTime == 0) return;
        let lastTime = Number(this.dictCache.vipStarTime);
        let disTime = (curTime - lastTime) / 1000 / 60;//1440为24小时的分钟
        if (disTime >= 1440) {//超过24小时
            this.vipCount = 0;
            this.dictCache.vipStarTime = 0;
        } else {//没到24小时
            const surTime = 1440 - disTime;
            setTimeout(() => {
                this.vipCount = 0;
                this.dictCache.vipStarTime = 0;
            }, surTime * 1000 * 60);
        }
    }

    /**
  * 恢复体力
  */
    recoveryPhysical(time?:number) {
        if (this.isCutDown) return;
        this.isCutDown = true;
        //如果体力小于最大值，每5分钟增加一个 ，并储存一个计时开始的时间
        if (!this.dictCache.curTime) {
            this.setTime();
        }
        setTimeout(() => {
            this.isCutDown = false;
            let count = this.physical;
            if (count < global.Physical) {
                this.setPhysical(global.physicalCount);
            }
        }, time ? time * 1000 : (global.physicalTime * 60) * 1000);
    }
    /**
      * 设置体力
      * @param num 获得的体力数
      * @param exceed 是否能够超越最大值
      */
    setPhysical(num: number, exceed: boolean = false) {
        let count = this.physical;
        count += num;
        this.dictCache.curTime = null!;
        //可以超越
        if (exceed) {
            this.dictCache.physical = count;
        } else {
            if (count > global.Physical) {
                this.dictCache.physical = global.Physical;
            } else {
                this.dictCache.physical = count;
                this.recoveryPhysical();
            }
        }
        this.emit(GDef.wdEvent.upPhysical)
        this.updateCache();
    }

    /**
       * 设置时间
       */
    setTime() {
        if (!this.dictCache.curTime) {
            const date = new Date();
            var curTime = date.getTime();
            this.dictCache.curTime = curTime;
        }
        this.updateCache();
    }
    /**
        * 获取上一次损耗体力开始时间
        */
    getTime() {
        if (!this.dictCache.curTime) {
            this.setTime();
        }
        return this.dictCache.curTime;
    }

    // /**判断需要弹出的弹窗类型 */
    // popupType(){
    //     if(GD.playerMgr.dictCache.popupState == 1){
    //         GD.playerMgr.dictCache.popupState = 2
    //     }else{
    //         GD.playerMgr.dictCache.popupState = 1
    //     }
    //     this.updateCache();
    // }


    /**
     * 设置游戏状态
     * 0:等待,1:继续,2:结束
     */
    setGameState(state: number): void {
        this.gameState = state;
    }

    /**
     * 获取游戏状态
     * 0:等待,1:继续,2:结束
     */
    getGameState(): number {
        return this.gameState;
    }

    /**获取金币 */
    get coin(): number {
        return this.dictCache.coin;
    }

    /**获取体力 */
    get physical(): number {
        return this.dictCache.physical;
    }

    /**设置金币 */
    setcoin(num: number) {
        this.dictCache.coin = num + Number(this.dictCache.coin);
        this.updateCache();
        this.emit(GDef.wdEvent.updateCoin);
    }

    /**
       * 消耗体力
       */
    set physical(num) {
        let count = this.physical;
        count += num;
        this.dictCache.physical = count;
        if (count >= global.Physical) {
            this.dictCache.curTime = null!;
        } else {
            this.recoveryPhysical();
        }
        this.updateCache();
    }
    set player(str: string) {
        this.dictCache.player = str;
        this.updateCache();
    }

    get player(): string {
        return this.dictCache.player;
    }

    /**设置声音 */
    set Voice(bool: boolean) {
        if (bool) {
            this.dictCache.voice = 1;
        } else {
            this.dictCache.voice = 2;
        }
        this.emit(GDef.wdEvent.voiceSet, this.Voice);
        this.updateCache();
    }

    /**获取声音设置状态 */
    get Voice(): boolean {
        const bool = this.dictCache.voice == 1 ? true : false;
        return bool;
    }

    /**设置震动 */
    set Shock(bool: boolean) {
        if (bool) {
            this.dictCache.shock = 1;
        } else {
            this.dictCache.shock = 2;
        }
        this.updateCache();
    }

    /**获取震动设置状态 */
    get Shock(): boolean {
        const bool = this.dictCache.shock == 1 ? true : false;
        return bool;
    }
    /**获得当前观看vip视频的次数 */
    get vipCount(): number {
        if (!this.dictCache.vipCount) {
            this.dictCache.vipCount = 0;
        }
        return this.dictCache.vipCount;
    }

    /**s设置观看次数 */
    set vipCount(num: number) {
        if (!this.dictCache.vipCount) {
            this.dictCache.vipCount = 0;
        }
        this.dictCache.vipCount = num;
        //如果当前vip次数达到，则倒计时24小时
        if (this.dictCache.vipCount >= global.vipCount) {
            const date = new Date();
            var curTime = date.getTime();
            this.dictCache.vipStarTime = curTime;
            //体力值满
            if (this.dictCache.physical < global.Physical) {
                this.dictCache.physical = global.Physical;
                this.emit(GDef.wdEvent.upPhysical);
            }
        }
    }

    /**获得地狱模式当前观看视频的次数 */
    get specialModelCount_1(): number {
        if (!this.dictCache.specialModelCount_1) {
            this.dictCache.specialModelCount_1 = 0;
        }
        return this.dictCache.specialModelCount_1;
    }

    /**设置地狱模式观看次数 */
    set specialModelCount_1(num: number) {
        if (!this.dictCache.specialModelCount_1) {
            this.dictCache.specialModelCount_1 = 0;
        }
        this.dictCache.specialModelCount_1 = num;
    }

     /**获得天选模式当前观看视频的次数 */
     get specialModelCount_2(): number {
        if (!this.dictCache.specialModelCount_2) {
            this.dictCache.specialModelCount_2 = 0;
        }
        return this.dictCache.specialModelCount_2;
    }

    /**设置天选模式观看次数 */
    set specialModelCount_2(num: number) {
        if (!this.dictCache.specialModelCount_2) {
            this.dictCache.specialModelCount_2 = 0;
        }
        this.dictCache.specialModelCount_2 = num;
    }
}

export default playerMgr;