
import { _decorator, Component, Node, Sprite, gfx, SpriteFrame, SystemEventType, System, SystemEvent, systemEvent, instantiate, Label, tween, v3 } from 'cc';
import { SDKbaseView } from './SDKbaseView';
import { tyqSDK } from './tyq-sdk';
const { ccclass, property } = _decorator;

@ccclass('TyqLuckDraw')
export class TyqLuckDraw extends SDKbaseView {

    @property({
        type: SpriteFrame,
        tooltip: "轮换的图片，第一张是默认，第二张是选中"
    })
    spList: SpriteFrame[] = [];

    @property({
        type: Node,
        tooltip: "抽奖的父节点"
    })
    layout: Node = null!;

    @property({
        type: Node,
        tooltip: "关闭按钮"
    })
    close: Node = null!;

    @property({
        type: Node,
        tooltip: "抽一次"
    })
    once: Node = null!;

    @property({
        type: Node,
        tooltip: "看视频抽一次"
    })
    videoOnce: Node = null!;

    @property({
        type: Node,
        tooltip: "奖励节点"
    })
    reward: Node = null!;

    private drawData = [
        { type: "phy", count: 2 },
        { type: "diam", count: 10 },
        { type: "null", count: 0 },
        { type: "phy", count: 5 },
        { type: "diam", count: 2 },
        { type: "phy", count: 3 },
        { type: "null", count: 0 },
        { type: "diam", count: 5 },
    ];
    private curIndex = 0;
    start() {
        //关闭按钮
        this.bindButton(this.close, () => {
            this.node.destroy();
        })
        this.bindButton(this.once, this.starDraw);//直接抽
        this.bindButton(this.videoOnce, this.videoDraw);//看视频抽
        //生成奖励对应的数量
        for (let i = 1; i < this.drawData.length; i++) {
            if (this.drawData[i].count == 0) continue;
            const node = instantiate(this.layout.children[0].getChildByName("Label"));
            node!.getComponent(Label)!.string = "x" + this.drawData[i].count;
            node!.parent = this.layout.children[i];
        }
        var curTime =tyqSDK.getTimeStamp(true);
        //一天只能免费抽奖一次
        const last = tyqSDK.getItem("tyq_draw");
        if (!last) {//没有抽过奖
            this.videoOnce.active = false;
        } else if (last == curTime) {//今天抽过奖
            this.once.active = false;
        } else {//今天没抽过奖
            this.videoOnce.active = false;
        }
        this.node.setSiblingIndex(10);
    }

    /**开始抽奖 */
    starDraw() {
        //按钮隐藏
        this.close.active = false;
        this.videoOnce.active = false;
        this.once.active = false;
        //开始随机一个
        const num = tyqSDK.RandomNumBoth(40, 50);
        console.log(num);
        console.log(this.curIndex);
        this.showDraw(this.curIndex, num);
        var data = new Date();
        const curTime = data.getFullYear().toString() + (data.getMonth() + 1).toString() + data.getDate().toString()//精确到日期  
        tyqSDK.setItem("tyq_draw", curTime);
    }

    /**转盘表现 */
    showDraw(curIndex: number, endIndex: number) {
        const index = curIndex % this.layout.children.length;
        this.curIndex = index;
        let lastIndex = index - 1;
        if (lastIndex < 0) {
            lastIndex = this.layout.children.length - 1;
        }

        this.layout.children[lastIndex].getComponent(Sprite)!.spriteFrame = this.spList[0];
        this.layout.children[index].getComponent(Sprite)!.spriteFrame = this.spList[1];
        //达到极限
        if (curIndex == endIndex) {
            if (this.drawData[index].count == 0) {
                this.endReward();
                return;
            };
            this.showReward(index);
            return;
        }
        let delyTime = 0;
        if (curIndex < (endIndex / 2)) {
            delyTime = 0.25 - curIndex / endIndex;

        } else {
            delyTime = curIndex / endIndex - 0.75;
        }
        if (delyTime < 0) {
            delyTime = 0.02;
        }
        this.scheduleOnce(() => {
            curIndex++;
            this.showDraw(curIndex, endIndex);
        }, delyTime)
    }

    /**
     * 展示奖励
     */
    showReward(index: number) {
        this.reward.destroyAllChildren();
        const uiNode = instantiate(this.layout.children[index].getChildByName("ui"));
        uiNode!.parent = this.reward;
        const labelNode = instantiate(this.layout.children[index].getChildByName("Label"));
        labelNode!.parent = this.reward;
        labelNode!.setPosition(0, -50, 0);
        this.reward.active = true;
        this.reward.setScale(0, 0, 0);
        tween(this.reward).to(0.5, { scale: v3(2, 2, 2) }).delay(1)
            .call(() => {
                this.endReward();
            })
            .start();
        if (this.drawData[index].type == "phy") {
            tyqSDK.Physical = this.drawData[index].count;
        } else {
            tyqSDK.Diamonds = this.drawData[index].count;
        }
    }

    /**结束奖励 */
    endReward() {
        this.reward.active = false;
        this.close.active = true;
        this.videoOnce.active = true;
    }

    /**看视频抽奖 */
    videoDraw() {
        tyqSDK.showVideoAD("今日抽奖", () => {
            this.starDraw();
        })
    }
}
