import { _decorator, Component, Node, SpriteFrame, SpriteComponent, Prefab, ButtonComponent, misc, instantiate } from 'cc';
import { SDKbaseView } from './SDKbaseView';
import { tyqSDK } from './tyq-sdk';
import { tyq_boxItem } from './tyq_boxItem';
const { ccclass, property } = _decorator;

@ccclass('tyq_luckyBox')
export class tyq_luckyBox extends SDKbaseView {

    @property({
        type: Node,
        tooltip: "关闭按钮"
    })
    closeBtn: Node = null!;
    @property({
        type: Node,
        tooltip: "看视频按钮按钮"
    })
    videoBtn: Node = null!;
    @property({
        type: Node,
        tooltip: "宝箱存放的父节点"
    })
    boxLayout: Node = null!;
    @property({
        type: Node,
        tooltip: "钥匙存放的父节点"
    })
    keyLayout: Node = null!;

    @property({
        type: Prefab,
        tooltip: "宝箱预制体"
    })
    tyq_boxItem: Prefab = null!;


    /**下一关回调 */
    private callback: Function | null = null;
    private canTouch: boolean = true;
    /**打开宝箱的个数 */
    private openNum = 0;
    private boxList: Array<number> = [];
    start() {
        this.bindButton(this.closeBtn, this.closeFun);
        this.bindButton(this.videoBtn, this.videoFun);
        this.initBox();
        this.videoBtn.parent!.active = false;
    }

    init(next: Function) {
        this.callback = next;
    }


    /**初始化宝箱 */
    initBox() {
        //随机3个 视频宝箱变成普通宝箱 这里随机3个变成普通宝箱 
        let arry = [0, 1, 2, 3, 4, 5, 6, 7, 8];
        let randList = [];//
        for (let i = 0; i < 3; i++) {
            const rand1 = tyqSDK.RandomNumBoth(0, arry.length - 1);
            const num1 = arry[rand1];
            arry.splice(rand1, 1);
            randList.push(num1);
        }
        for (let i = 0; i < 9; i++) {
            this.boxList.push(i);
            let type = true;//视频宝箱
            for (let j = 0; j < randList.length; j++) {
                if (i == randList[j]) {
                    type = false;//普通宝箱
                    break;
                }
            }
            this.showUI(this.tyq_boxItem, this.boxLayout, (comp: tyq_boxItem) => {
                comp.init(type);
                this.bindButton(comp.node, this.touchBox, i, !type);
            })
        }
    }


    /**点击宝箱 */
    touchBox(index: number, isDefault: boolean) {
        if (!this.canTouch) return;
        this.canTouch = false;

        if (isDefault) {
            this.openBox(index);
        } else {
            tyqSDK.showVideoAD("视频宝箱", () => {
                this.openBox(index);
            }, () => {
                this.canTouch = true;
            })
        }
    }
    /**打开宝箱
     * @param isDefault 是否是普通宝箱
     * @param index 宝箱编号
     */
    openBox(index: number) {
        this.canTouch = true;
        const node = this.boxLayout.children[index];
        node.getComponent(ButtonComponent)!.enabled = false;
        const boxTs = node.getComponent(tyq_boxItem)!
        const isDefault = boxTs.isWatch;
        boxTs.open();
        //普通宝箱移除钥匙
        if (!isDefault) {
            this.keyLayout.children[0]!.destroy();
        }
        this.openNum++;
        if (this.openNum >= 3) {//开启超过3个显示关闭按钮
            this.videoBtn.parent!.active = true;
        }
        if (this.openNum >= 9) {//开启超过9个隐藏看视频按钮
            this.videoBtn.active = false;
        }
        for (let i = 0; i < this.boxList.length; i++) {
            if (index == this.boxList[i]) {
                this.boxList.splice(i, 1);
                break;
            }
        }
    }

    /**右上角的关闭按钮 */
    closeFun() {
        this.next();
    }

    /**
     * 看视频
     */
    videoFun() {
        if (!this.canTouch) return;
        this.canTouch = false;
        tyqSDK.showVideoAD("再开一个", () => {
            const rand = tyqSDK.RandomNumBoth(0, this.boxList.length - 1);
            const index = this.boxList[rand];
            this.openBox(index);
        }, () => {
            this.canTouch = true;
        })
    }

    /**离开幸运宝箱页面 */
    next() {
        if (this.callback) {
            this.callback();
        }
        this.node.destroy();
    }



    // update (deltaTime: number) {
    //     // Your update function goes here.
    // }
}
